.result_header {
    margin-top: 0px;
    margin-bottom: 2px;
    font-size: 16px;

    .favicon {
        margin-bottom:-3px;
    }

    a {
        color: @black;
        text-decoration: none;

        &:hover{
            color: @blue;
        }

        &:visited{
            color: @violet;
        }

        .highlight {
            background-color: @dim-gray;
            // Chrome hack: bold is different size than normal
            // https://stackoverflow.com/questions/20713988/weird-text-alignment-issue-in-css-when-bolded-lucida-sans
        }
    }
}

.result-content, .result-format, .result-source {
    margin-top: 2px;
    margin-bottom: 0;
    word-wrap: break-word;
    color: @dark-gray;
    font-size: 13px;
}

.result .highlight {
    font-weight: bold;
}

.result-source {
    font-size: 10px;
    float: left;
}

.result-format {
    font-size: 10px;
    float: right;
}

.result-abstract {
    margin-top: 0.5em;
    margin-bottom: 0.8em;
}

.external-link {
    color: @dark-green;
    font-size: 12px;
    margin-bottom: 15px;

    a {
        margin-right: 3px;
    }
}

// default formating of results
.result-default, .result-code, .result-torrent, .result-videos, .result-map {
    clear: both;
    padding: 0.5em 4px;
    &:hover{
        background-color: @dim-gray;
    }
}


// image formating of results
.result-images {
    float: left !important;
    margin: 0;
    padding: 0;
    a {
        display: block;
        width: 100%;
        background-size: cover;
        .img-thumbnail {
            border: none !important;
            padding: 0;
        }
        &:hover, &:focus {
            outline: 0;
            .img-thumbnail {
                box-shadow: 5px 5px 15px 0px black;
            }
        }
    }
}

.result-images.js a .img-thumbnail {
    max-height: inherit;
    min-height: inherit;
}

.result-images:not(.js) {
    width: 25%;
    padding: 3px 13px 13px 3px;
    a {
        .img-thumbnail {
            margin: 0;
            max-height: 128px;
            min-height: 128px;
        }
    }
}

.img-thumbnail {
    margin: 5px;
    max-height: 128px;
    min-height: 128px;
}

// video formating of results
.result-videos {
    clear: both;

    hr{
        margin: 5px 0 15px 0;
    }

    .collapse{
        width: 100%;
    }

    .in{
        margin-bottom: 8px;
    }
}

// torrent formating of results
.result-torrent {
    clear: both;

    b{
        margin-right: 5px;
        margin-left: 5px;
    }

    .seeders{
        color: @green;
    }

    .leechers{
        color: @red;
    }
}

.result-metadata {
    clear: both;
    margin: 1em;

    td {
        padding-right: 1em;
        color: @gray;
    }

    td:first-of-type {
        color: @dark-gray;
    }
}

// map formating of results
.result-map {
    clear: both;

    .img-thumbnail {
        float: right;
        width: auto;
        height: 120px;
        border: 0;
        background: inherit;
    }

    .img-type {
        width: 20px;
        max-height: 20px;
    }

    .result-map-details {
        font-size: 13px;
        border-collapse: separate;
        border-spacing: 0 0.35rem;

        th {
            font-weight: inherit;
            width: 20rem;
            vertical-align: top;
        }

        td {
            vertical-align: top;
        }
    }

}

// code formating of results
.result-code {
    clear: both;

    .code-fork, .code-fork a{
        color: @dark-gray;
    }

}

// suggestion
.suggestion_item {
    margin: 2px 5px;
    max-width: 100%;

    .btn {
        max-width: 100%;
        white-space: normal;
        word-wrap: break-word;
        text-align: left;
    }
}

// download result
.result_download {
    margin-right: 5px;
}

// page forward, backward
#pagination {
    margin-top: 30px;
    padding-bottom: 60px;
}

.label-default {
    color: @dark-gray;
    background: transparent;
}

.result .text-muted small {
    word-wrap: break-word;
}

.modal-wrapper {
    box-shadow: 0 5px 15px rgba(0, 0, 0, 0.5);
}

.modal-wrapper {
    background-clip: padding-box;
    background-color: #fff;
    border: 1px solid rgba(0, 0, 0, 0.2);
    border-radius: 6px;
    box-shadow: 0 3px 9px rgba(0, 0, 0, 0.5);
    outline: 0 none;
    position: relative;
}

@media screen and (max-width: 75em) {
    .img-thumbnail {
        object-fit: cover;
    }
}
