.searx-navbar {
    background: @black;
    height: 2.3rem;
    font-size: 1.3rem;
    line-height: 1.3rem;
    padding: 0.5rem;
    font-weight: bold;
    margin-bottom: 0.8rem;

    a, a:hover {
        margin-right: 2.0rem;
        color: white;
        text-decoration: none;
    }

    .instance a {
        color: @light-green;
        margin-left: 2.0rem;
    }
}

#main-logo {
    margin-top: 20vh;
    margin-bottom: 25px;

    & > img {
        max-width: 350px;
        width: 80%;
    }
}
