<div class="alert alert-info fade in" role="alert">
    <button class="close" data-dismiss="alert" type="button">
        <span aria-hidden="true">×</span>
        <span class="sr-only">{{ _('Close') }}</span>
    </button>
    <strong class="lead">{{ icon('info-sign') }} {{ _('Heads up!') }}</strong>
    {{ _('It look like you are using searx first time.') }}
</div>
