{% from 'oscar/macros.html' import icon %}
<form method="{{ method or 'POST' }}" action="{{ url_for('search') }}" id="search_form" role="search">
  <div class="row">
    <div class="col-xs-12 col-md-8">
      <div class="input-group search-margin">
        <input type="search" {% if autofocus %}autofocus{% endif %} name="q" class="form-control" id="q" placeholder="{{ _('Search for...') }}" aria-label="{{ _('Search for...') }}" autocomplete="off" value="{{ q }}" accesskey="s">
        <span class="input-group-btn">
            <button type="submit" class="btn btn-default" aria-label="{{ _('Start search') }}"><span class="hide_if_nojs">{{ icon('search') }}</span><span class="hidden active_if_nojs">{{ _('Start search') }}</span></button>
	    <button type="button" id="clear_search" class="btn btn-default hide_if_nojs" aria-label="{{ _('Clear search') }}">{{ icon('remove') }}</button>
        </span>
      </div>
    </div>
    <div class="col-xs-6 col-md-2 search-margin">
        {%- include 'oscar/time-range.html' -%}
    </div>
    <div class="col-xs-6 col-md-2 search-margin">
        {%- include 'oscar/languages.html' -%}
    </div>
  </div>
  <div class="row">
    <div class="col-sm-12">
        {%- include 'oscar/categories.html' -%}
    </div>
  </div>
</form><!-- / #search_form_full -->
