<table>
    {% for key, value in result.items() %}
    {% if key in ['engine', 'engines', 'template', 'score', 'category', 'positions'] %}
        {% continue %}
    {% endif %}
    <tr>
        <td><b>{{ key|upper }}</b>: {{ value }}</td>
    </tr>
    {% endfor %}
</table>
<div class="engines">{% for engine in result.engines %}<span>{{ engine }}</span>{% endfor %}</div>{{- '' -}}
