/* Generated by grunt-webfont */


@font-face {
	font-family:"ion";
	src:url("../fonts/ion.eot?ce7a0ead692560b4405a96d5b8471f51");
	src:url("../fonts/ion.eot?#iefix") format("embedded-opentype"),
		url("../fonts/ion.woff2?ce7a0ead692560b4405a96d5b8471f51") format("woff2"),
		url("../fonts/ion.woff?ce7a0ead692560b4405a96d5b8471f51") format("woff"),
		url("../fonts/ion.ttf?ce7a0ead692560b4405a96d5b8471f51") format("truetype"),
		url("../fonts/ion.svg?ce7a0ead692560b4405a96d5b8471f51#ion") format("svg");
	font-weight:normal;
	font-style:normal;
}

.ion-icon {
	
		font-family:"ion";
	
	display:inline-block;
	vertical-align:middle;
	line-height:1;
	font-weight:normal;
	font-style:normal;
	speak:none;
	text-decoration:inherit;
	text-transform:none;
	text-rendering:auto;
	-webkit-font-smoothing:antialiased;
	-moz-osx-font-smoothing:grayscale;
}


/* Icons */


.ion-navicon-round:before {
	content:"\f101";
}


.ion-search:before {
	content:"\f102";
}


.ion-play:before {
	content:"\f103";
}


.ion-link:before {
	content:"\f104";
}


.ion-chevron-up:before {
	content:"\f105";
}


.ion-chevron-left:before {
	content:"\f106";
}


.ion-chevron-right:before {
	content:"\f107";
}


.ion-arrow-down-a:before {
	content:"\f108";
}


.ion-arrow-up-a:before {
	content:"\f109";
}


.ion-arrow-swap:before {
	content:"\f10a";
}


.ion-arrow-dropdown:before {
	content:"\f10b";
}


.ion-globe:before {
	content:"\f10c";
}


.ion-time:before {
	content:"\f10d";
}


.ion-location:before {
	content:"\f10e";
}


.ion-warning:before {
	content:"\f10f";
}


.ion-error:before {
	content:"\f110";
}


.ion-film-outline:before {
	content:"\f111";
}


.ion-music-note:before {
	content:"\f112";
}


.ion-more-vertical:before {
	content:"\f113";
}


.ion-magnet:before {
	content:"\f114";
}


.ion-close:before {
	content:"\f115";
}
