{% from 'oscar/macros.html' import result_footer, result_footer_rtl with context %}
<div class="result result-default">
<table class="table table-responsive table-bordered table-condensed">
    {% for key, value in result.items() %}
    {% if key in ['engine', 'engines', 'template', 'score', 'category', 'positions', 'pretty_url', 'parsed_url'] %}
        {% continue %}
    {% endif %}
    <tr>
        <td><b>{{ key|upper }}</b>: {{ value|truncate }}</td>
    </tr>
    {% endfor %}
</table>

{% if rtl %}
{{ result_footer_rtl(result) }}
{% else %}
{{ result_footer(result) }}
{% endif %}
</div>
