{% from 'oscar/macros.html' import icon %}
<div class="alert alert-success fade in" role="alert">
    <button class="close" data-dismiss="alert" type="button">
        <span aria-hidden="true">×</span>
        <span class="sr-only">{{ _('Close') }}</span>
    </button>
    <strong class="lead">{{ icon('ok-sign') }} {{ _('Well done!') }}</strong>
    {{ _('Settings saved successfully.') }}
</div>
