.search_categories, #categories {
  text-transform: capitalize;
  margin-bottom: 0.5rem;
  display: flex;
  flex-wrap: wrap;
  flex-flow: row wrap;
  align-content: stretch;

  a {
    height: 3rem;
    flex-grow: 1;
    flex-basis: auto;
    border: @mild-gray 1px solid;
    border-right: none;
    color: @dark-gray;
    padding-bottom: 0.3rem;
    padding-top: 0.1rem;
    text-align: center;
    min-width: 50px;
  }

  label, .input-group-addon {
    flex-grow: 1;
    flex-basis: auto;
    font-size: 1.2rem;
    font-weight: normal;
    background-color: white;
    border: none;
    border-right: none;
    color: @dark-gray;
    padding-bottom: 0;
    padding-top: 0;
    text-align: center;
    width: 100%;
    height: 100%;
  }

  a:last-child, .input-group-addon:last-child {
      border-right: @mild-gray 1px solid;
  }

  input[type="checkbox"]:checked + a {
    color: @black;
    font-weight: bold;
    border-bottom: @light-green 5px solid;
  }
}

#main-logo{
    margin-top: 10vh;
    margin-bottom: 25px;
}

#main-logo > img {
    max-width: 350px;
    width: 80%;
}

#q{
    box-shadow: none;
    border-right: none;
    border-color: @semidark-gray;
}

 #search_form .input-group-btn .btn{
    border-color: @semidark-gray;
 }

 #search_form .input-group-btn .btn:hover{
     background-color: @dark-green;
     color: white;
 }

.custom-select, .custom-select-rtl {
    appearance: none;
    -webkit-appearance: none;
    -moz-appearance: none;
    font-size: 1.2rem;
    font-weight:normal;
    background-color: white;
    border: @semidark-gray 1px solid;
    color: @dark-gray;
    // Warning: multi-line base64 URL adds "$stdin" in the source map: https://github.com/jakubpawlowicz/clean-css/issues/593
    background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAA8AAAAPCAQAAACR313BAAAABGdBTUEAALGPC/xhBQAAACBjSFJNAAB6JgAAgIQAAPoAAACA6AAAdTAAAOpgAAA6mAAAF3CculE8AAAAAmJLR0QA/4ePzL8AAAAJcEhZcwAABFkAAARZAVnbJUkAAAAHdElNRQfgBxgLDwB20OFsAAAAbElEQVQY073OsQ3CMAAEwJMYwJGnsAehpoXJItltBkmcdZBYgIIiQoLglnz3ui+eP+bk5uneteTMZJa6OJuIqvYzSJoqwqBq8gdmTTW86/dghxAUq4xsVYT9laBYXCw93Aajh7GPEF23t4fkBYevGFTANkPRAAAAJXRFWHRkYXRlOmNyZWF0ZQAyMDE2LTA3LTI0VDExOjU1OjU4KzAyOjAwRFqFOQAAACV0RVh0ZGF0ZTptb2RpZnkAMjAxNi0wNy0yNFQxMToxNTowMCswMjowMP7RDgQAAAAZdEVYdFNvZnR3YXJlAHd3dy5pbmtzY2FwZS5vcmeb7jwaAAAAAElFTkSuQmCC) 96% no-repeat;
    padding-right: 26px;
    text-overflow: ellipsis;
}

.custom-select-rtl {
    background-position-x: 4%;
}

.search-margin {
    margin-bottom: 0.6em;
}

.visually-hidden {
    position: absolute !important;
    height: 1px;
    width: 1px;
    overflow: hidden;
    clip: rect(1px, 1px, 1px, 1px);
    white-space: nowrap;
}
.label-danger, .btn-danger {
    background: @dark-red;
}
.label-success, .btn-success {
    background: @dark-green;
}
select.form-control {
    border-color: @semidark-gray !important;
}
