.vim-hotkeys-help {
    position: fixed;
    top: 50%;
    left: 50%;
    transform: translate(-50%, -50%);
    z-index: 9999999;
    overflow-y: auto;
    max-height: 80%;
    box-shadow: 0 0 1em;
}

.dflex {
    display: -webkit-box;  /* OLD - iOS 6-, Safari 3.1-6 */
    display: -moz-box;     /* OLD - Firefox 19- (buggy but mostly works) */
    display: -ms-flexbox;  /* TWEENER - IE 10 */
    display: -webkit-flex; /* NEW - Chrome */
    display: flex;         /* NEW, Spec - Opera 12.1, Firefox 20+ */
}

.iflex {
    -webkit-box-flex: 1; /* OLD - iOS 6-, Safari 3.1-6 */
    -moz-box-flex: 1;    /* OLD - Firefox 19- */
    -webkit-flex: 1;     /* Chrome */
    -ms-flex: 1;         /* IE 10 */
    flex: 1;             /* NEW, Spec - Opera 12.1, Firefox 20+ */
}
