{% from 'oscar/macros.html' import icon %}
<!DOCTYPE html>
<html lang="{{ preferences.get_value('locale') }}" xml:lang="{{ preferences.get_value('locale') }}"{% if rtl %} dir="rtl"{% endif %} class="nojs">
<head>
    <meta charset="UTF-8" />
    <meta name="description" content="searx - a privacy-respecting, hackable metasearch engine" />
    <meta name="keywords" content="searx, search, search engine, metasearch, meta search" />
    <meta name="generator" content="searx/{{ searx_version }}">
    <meta name="referrer" content="no-referrer">
    <meta name="viewport" content="width=device-width, initial-scale=1 , maximum-scale=2.0, user-scalable=1" />
    {% block meta %}{% endblock %}
    <title>{% block title %}{% endblock %}{{ instance_name }}</title>
    <link rel="stylesheet" href="{{ url_for('static', filename='css/bootstrap.min.css') }}" type="text/css" />
    {% if preferences.get_value('oscar-style') -%}
    {{'    '}}<link rel="stylesheet" href="{{ url_for('static', filename='css/'+preferences.get_value('oscar-style')+'.min.css') }}" type="text/css" />
    {%- else -%}
    {{'    '}}<link rel="stylesheet" href="{{ url_for('static', filename='css/logicodev.min.css') }}" type="text/css" />
    {%- endif %}

    {% set templates = results|map(attribute='template')|unique|list -%}
    {%- set load_leaflet = 'map.html' in templates -%}
    {%- if load_leaflet -%}
    {{'    '}}<link rel="stylesheet" href="{{ url_for('static', filename='css/leaflet.min.css') }}" type="text/css" />
    {%- endif %}

    {%- for css in styles %}
        <link rel="stylesheet" href="{{ url_for('static', filename=css) }}" type="text/css" />
    {% endfor %}

    <link rel="shortcut icon" href="{{ url_for('static', filename='img/favicon.png') }}" />

    {% block styles %}
    {% endblock %}
    {% block head %}
    {% endblock %}

    <link title="{{ instance_name }}" type="application/opensearchdescription+xml" rel="search" href="{{ opensearch_url }}"/>
    <noscript>
        <style type="text/css">
            .tab-content > .active_if_nojs, .active_if_nojs {display: block !important; visibility: visible !important;}
            .margin_top_if_nojs {margin-top: 20px;}
            .hide_if_nojs {display: none !important;overflow: hidden !important;}
            .disabled_if_nojs {pointer-events: none; cursor: default; text-decoration: line-through;}
        </style>
    </noscript>
</head>
<body class="{{ endpoint }}_endpoint" >
    {% include 'oscar/navbar.html' %}

    <div class="container">
    {% if errors %}
        <div class="alert alert-danger fade in" role="alert">
            <button class="close" data-dismiss="alert" type="button">
                <span aria-hidden="true">×</span>
                <span class="sr-only">{{ _('Close') }}</span>
            </button>
            <strong class="lead">{{ icon('info-sign') }} {{ _('Error!') }}</strong>
            <ul>
            {% for message in errors %}
                <li>{{ message }}</li>
            {% endfor %}
            </ul>
        </div>
    {% endif %}

    {% block site_alert_error %}
    {% endblock %}
    {% block site_alert_warning %}
    {% endblock %}
    {% block site_alert_info %}
    {% endblock %}
    {% block site_alert_success %}
    {% endblock %}

    {% block content %}
    {% endblock %}

    </div>
    <div class="footer">
        <div class="container">
            {% block footer %}
            {% endblock %}
            <p class="text-muted">
                <small>
                    {{ _('Powered by') }} <a href="{{ brand.DOCS_URL }}">searx</a> - {{ searx_version }} - {{ _('a privacy-respecting, hackable metasearch engine') }}<br/>
                    <a href="{{ brand.GIT_URL }}">{{ _('Source code') }}</a> |
                    <a href="{{ brand.ISSUE_URL }}">{{ _('Issue tracker') }}</a> |
                    <a href="{{ brand.PUBLIC_INSTANCES }}">{{ _('Public instances') }}</a>{% if brand.CONTACT_URL %} |
                    <a href="{{ brand.CONTACT_URL }}">{{ _('Contact instance maintainer') }}</a>{% endif %}
                </small>
            </p>
        </div>
    </div>
    <script src="{{ url_for('static', filename='js/jquery.min.js') }}"></script>
    <script src="{{ url_for('static', filename='js/bootstrap.min.js') }}"></script>
    {% if load_leaflet %}{{"    "}}<script src="{{ url_for('static', filename='js/leaflet.js') }}"></script>{{ "\n" }}{% endif %}
    {% if autocomplete %}{{"    "}}<script src="{{ url_for('static', filename='js/typeahead.bundle.min.js') }}"></script>{{ "\n" }}{% endif %}

    <script src="{{ url_for('static', filename='js/searx.min.js') }}"
            data-method="{{ method or 'POST' }}"
            data-autocompleter="{% if autocomplete %}true{% else %}false{% endif %}"
            data-translations="{{ translations }}"></script>
    {% for script in scripts %}
    {{""}}<script src="{{ url_for('static', filename=script) }}"></script>
    {% endfor %}
    <noscript>
      <style>
        .glyphicon { display: none; }
      </style>
    </noscript>
</body>
</html>
