{% from 'oscar/macros.html' import draw_favicon, icon, result_header, result_sub_header, result_footer_rtl, result_footer %}

{{ result_header(result, favicons) }}
{{ result_sub_header(result) }}

<div class="container-fluid">
    <div class="row">
        <a href="{{ result.url }}" {% if results_on_new_tab %}target="_blank" rel="noopener noreferrer"{% else %}rel="noreferrer"{% endif %}><img class="thumbnail col-xs-6 col-sm-3 col-md-3 result-content" src="{{ image_proxify(result.thumbnail) }}" alt="{{ result.title|striptags }} {{ result.engine }}" /></a>
        <p class="col-xs-12 col-sm-9 col-md-9 result-content">
        {% if result.price %}<big>{{ result.price|safe }}</big></br>{% endif %}
        {% if result.shipping %}<small>{{ result.shipping|safe }}</small></br>{% endif %}
        {% if result.source_country %}<small>{{ result.source_country|safe }}</small></br>{% endif %}
        {% if result.content %}{{ result.content|safe }}{% endif %}
        {% if result.has_stock is defined %}<br>{% if result.has_stock %}{{ icon('check', _('Has stock')) }}{% else %}{{ icon('alert', _('Out of stock')) }}{% endif %}{% endif %}
        </p>
    </div>
</div>

{% if rtl %}
{{ result_footer_rtl(result) }}
{% else %}
{{ result_footer(result) }}
{% endif %}
