.searx-navbar {
    background: #eee;
    color: #aaa;
    height: 2.3rem;
    font-size: 1.3rem;
    line-height: 1.3rem;
    padding: 0.5rem;
    font-weight: bold;
    margin-bottom: 1.3rem;

    a, a:hover {
        margin-right: 2.0rem;
        text-decoration: none;
    }

    .instance a {
        color: #444;
        margin-left: 2.0rem;
    }
}
