.search_categories, #categories {
    text-transform: capitalize;
    margin-bottom: 1.5rem;
    margin-top: 1.5rem;
    display: flex;
    flex-wrap: wrap;
    align-content: stretch;

  a {
    flex-grow: 1;
    flex-basis: auto;
    border: @mild-gray 1px solid;
    border-right: none;
    color: @dark-gray;
    padding-bottom: 0.4rem;
    padding-top: 0.4rem;
    text-align: center;
    min-width: 50px;
  }

  label, .input-group-addon {
    flex-grow: 1;
    flex-basis: auto;
    font-size: 1.3rem;
    font-weight: normal;
    background-color: white;
    border: #DDD 1px solid;
    border-right: none;
    color: #333;
    padding-bottom: 0.8rem;
    padding-top: 0.8rem;
    text-align: center;
    width: 100%;
    height: 100%;
  }

  a:last-child, .input-group-addon:last-child {
      border-right: #DDD 1px solid;
  }

    input[type="checkbox"]:checked + label{
        color: black;
        font-weight: bold;
        background-color: #EEE;
    }
}

.visually-hidden {
    position: absolute !important;
    height: 1px;
    width: 1px;
    overflow: hidden;
    clip: rect(1px 1px 1px 1px); /* IE6, IE7 */
    clip: rect(1px, 1px, 1px, 1px);
    white-space: nowrap; /* added line */
}
