{% from 'oscar/macros.html' import icon %}
<div class="alert alert-info fade in" role="alert">
    <strong class="lead">{{ icon('info-sign') }} {{ _('Heads up!') }}</strong>
    {{ _('There is currently no data available. ') }}
</div>
