{% from 'simple/macros.html' import result_header, result_sub_header, result_sub_footer, result_footer, result_footer_rtl %}

{{ result_header(result, favicons, image_proxify) }}
{{ result_sub_header(result) }}
{% if result.embedded -%}
<p class="altlink"> &bull; <a class="btn-collapse collapsed media-loader disabled_if_nojs" data-target="#result-video-{{ index }}" data-btn-text-collapsed="{{ _('show video') }}" data-btn-text-not-collapsed="{{ _('hide video') }}">{{ icon('film-outline') }} {{ _('show video') }}</a></p>
{%- endif %}
{% if result.content %}<p class="content">{{ result.content|safe }}</p>{% endif %}
{{- result_sub_footer(result, proxify) -}}
{% if result.embedded -%}
<div id="result-video-{{ index }}" class="invisible">
   {{- result.embedded|safe -}}
</div>
{%- endif %}
{{ result_footer(result) }}
