html {
  position: relative;
  min-height: 100%;
}
body {
  /* Margin bottom by footer height */
  margin-bottom: 80px;
}
.footer {
  position: absolute;
  bottom: 0;
  width: 100%;
  /* Set the fixed height of the footer here */
  height: 60px;
}
input[type=checkbox]:checked + .label_hide_if_checked,
input[type=checkbox]:checked + .label_hide_if_not_checked + .label_hide_if_checked {
  display: none;
}
input[type=checkbox]:not(:checked) + .label_hide_if_not_checked,
input[type=checkbox]:not(:checked) + .label_hide_if_checked + .label_hide_if_not_checked {
  display: none;
}
.onoff-checkbox {
  width: 15%;
}
.onoffswitch {
  position: relative;
  width: 110px;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
}
.onoffswitch-checkbox {
  display: none;
}
.onoffswitch-label {
  display: block;
  overflow: hidden;
  cursor: pointer;
  border: 2px solid #FFFFFF !important;
  border-radius: 50px !important;
}
.onoffswitch-inner {
  display: block;
  transition: margin 0.3s ease-in 0s;
}
.onoffswitch-inner:before,
.onoffswitch-inner:after {
  display: block;
  float: left;
  width: 50%;
  height: 30px;
  padding: 0;
  line-height: 40px;
  font-size: 20px;
  box-sizing: border-box;
  content: "";
  background-color: #EEEEEE;
}
.onoffswitch-switch {
  display: block;
  width: 37px;
  background-color: #00CC00;
  position: absolute;
  top: 0;
  bottom: 0;
  right: 0px;
  border: 2px solid #FFFFFF !important;
  border-radius: 50px !important;
  transition: all 0.3s ease-in 0s;
}
.onoffswitch-checkbox:checked + .onoffswitch-label .onoffswitch-inner {
  margin-right: 0;
}
.onoffswitch-checkbox:checked + .onoffswitch-label .onoffswitch-switch {
  right: 71px;
  background-color: #A1A1A1;
}
.result_header {
  margin-bottom: 5px;
  margin-top: 20px;
}
.result_header .favicon {
  margin-bottom: -3px;
}
.result_header a {
  vertical-align: bottom;
}
.result_header a .highlight {
  font-weight: bold;
}
.result-content {
  margin-top: 5px;
  word-wrap: break-word;
}
.result-content .highlight {
  font-weight: bold;
}
.result-default {
  clear: both;
}
.result-images {
  float: left !important;
  height: 138px;
}
.img-thumbnail {
  margin: 5px;
  max-height: 128px;
}
.result-videos {
  clear: both;
}
.result-torrents {
  clear: both;
}
.result-map {
  clear: both;
}
.result-code {
  clear: both;
}
.suggestion_item {
  margin: 2px 5px;
  max-width: 100%;
}
.suggestion_item .btn {
  max-width: 100%;
  white-space: normal;
  word-wrap: break-word;
  text-align: left;
}
.result_download {
  margin-right: 5px;
}
#pagination {
  margin-top: 30px;
  padding-bottom: 50px;
}
.label-default {
  color: #AAA;
  background: #FFF;
}
.result .text-muted small {
  word-wrap: break-word;
}
.modal-wrapper {
  box-shadow: 0 5px 15px rgba(0, 0, 0, 0.5);
}
.modal-wrapper {
  background-clip: padding-box;
  background-color: #fff;
  border: 1px solid rgba(0, 0, 0, 0.2);
  border-radius: 6px;
  box-shadow: 0 3px 9px rgba(0, 0, 0, 0.5);
  outline: 0 none;
  position: relative;
}
.highlight {
  font-weight: 700;
}
.infobox img {
  max-height: 250px;
}
.infobox .infobox_part {
  margin-bottom: 20px;
  word-wrap: break-word;
  table-layout: fixed;
}
.infobox .infobox_part:last-child {
  margin-bottom: 0;
}
.infobox .header_url {
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
  display: block;
}
.infobox .infobox_toggle {
  width: 100%;
  text-align: center;
  margin-bottom: 0px;
}
.infobox .infobox_checkbox ~ .infobox_body {
  max-height: 300px;
  overflow: hidden;
}
.infobox .infobox_checkbox:checked ~ .infobox_body {
  max-height: none;
}
.infobox .infobox_checkbox ~ .infobox_toggle .infobox_label_down {
  display: block;
}
.infobox .infobox_checkbox ~ .infobox_toggle .infobox_label_up {
  display: none;
}
.infobox .infobox_checkbox:checked ~ .infobox_toggle .infobox_label_up {
  display: block;
}
.infobox .infobox_checkbox:checked ~ .infobox_toggle .infobox_label_down {
  display: none;
}
.infobox .infobox_checkbox ~ .infobox_body img.infobox_part {
  display: none;
}
.infobox .infobox_checkbox:checked ~ .infobox_body img.infobox_part {
  display: block;
}
.search_categories,
#categories {
  text-transform: capitalize;
  margin-bottom: 1.5rem;
  margin-top: 1.5rem;
  display: flex;
  flex-wrap: wrap;
  align-content: stretch;
}
.search_categories a,
#categories a {
  flex-grow: 1;
  flex-basis: auto;
  border: #DDD 1px solid;
  border-right: none;
  color: #666;
  padding-bottom: 0.4rem;
  padding-top: 0.4rem;
  text-align: center;
  min-width: 50px;
}
.search_categories label,
#categories label,
.search_categories .input-group-addon,
#categories .input-group-addon {
  flex-grow: 1;
  flex-basis: auto;
  font-size: 1.3rem;
  font-weight: normal;
  background-color: white;
  border: #DDD 1px solid;
  border-right: none;
  color: #333;
  padding-bottom: 0.8rem;
  padding-top: 0.8rem;
  text-align: center;
  width: 100%;
  height: 100%;
}
.search_categories a:last-child,
#categories a:last-child,
.search_categories .input-group-addon:last-child,
#categories .input-group-addon:last-child {
  border-right: #DDD 1px solid;
}
.search_categories input[type="checkbox"]:checked + label,
#categories input[type="checkbox"]:checked + label {
  color: black;
  font-weight: bold;
  background-color: #EEE;
}
.visually-hidden {
  position: absolute !important;
  height: 1px;
  width: 1px;
  overflow: hidden;
  clip: rect(1px 1px 1px 1px);
  /* IE6, IE7 */
  clip: rect(1px, 1px, 1px, 1px);
  white-space: nowrap;
  /* added line */
}
#advanced-search-container {
  display: none;
  text-align: center;
  margin-bottom: 1rem;
  clear: both;
}
#advanced-search-container label,
#advanced-search-container .input-group-addon {
  font-size: 1.3rem;
  font-weight: normal;
  background-color: white;
  border: #DDD 1px solid;
  border-right: none;
  color: #333;
  padding-bottom: 0.8rem;
  padding-left: 1.2rem;
  padding-right: 1.2rem;
}
#advanced-search-container label:last-child,
#advanced-search-container .input-group-addon:last-child {
  border-right: #DDD 1px solid;
}
#advanced-search-container input[type="radio"] {
  display: none;
}
#advanced-search-container input[type="radio"]:checked + label {
  color: black;
  font-weight: bold;
  background-color: #EEE;
}
#check-advanced {
  display: none;
}
#check-advanced:checked ~ #advanced-search-container {
  display: block;
}
.advanced {
  padding: 0;
  margin-top: 0.3rem;
  text-align: right;
}
.advanced label,
.advanced select {
  cursor: pointer;
}
.cursor-text {
  cursor: text !important;
}
.cursor-pointer {
  cursor: pointer !important;
}
/*
   this file is generated automatically by searx_extra/update/update_pygments.py 
   using pygments version 2.8.0
*/
.code-highlight .linenos {
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -khtml-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
  cursor: default;
  margin-right: 8px;
  text-align: right;
}
.code-highlight .linenos::selection {
  background: transparent;
  /* WebKit/Blink Browsers */
}
.code-highlight .linenos::-moz-selection {
  background: transparent;
  /* Gecko Browsers */
}
.code-highlight pre {
  line-height: 125%;
}
.code-highlight td.linenos .normal {
  color: inherit;
  background-color: transparent;
  padding-left: 5px;
  padding-right: 5px;
}
.code-highlight span.linenos {
  color: inherit;
  background-color: transparent;
  padding-left: 5px;
  padding-right: 5px;
}
.code-highlight td.linenos .special {
  color: #000000;
  background-color: #ffffc0;
  padding-left: 5px;
  padding-right: 5px;
}
.code-highlight span.linenos.special {
  color: #000000;
  background-color: #ffffc0;
  padding-left: 5px;
  padding-right: 5px;
}
.code-highlight .hll {
  background-color: #ffffcc;
}
.code-highlight {
  background: #f8f8f8;
}
.code-highlight .c {
  color: #408080;
  font-style: italic;
}
/* Comment */
.code-highlight .err {
  border: 1px solid #FF0000;
}
/* Error */
.code-highlight .k {
  color: #008000;
  font-weight: bold;
}
/* Keyword */
.code-highlight .o {
  color: #666666;
}
/* Operator */
.code-highlight .ch {
  color: #408080;
  font-style: italic;
}
/* Comment.Hashbang */
.code-highlight .cm {
  color: #408080;
  font-style: italic;
}
/* Comment.Multiline */
.code-highlight .cp {
  color: #BC7A00;
}
/* Comment.Preproc */
.code-highlight .cpf {
  color: #408080;
  font-style: italic;
}
/* Comment.PreprocFile */
.code-highlight .c1 {
  color: #408080;
  font-style: italic;
}
/* Comment.Single */
.code-highlight .cs {
  color: #408080;
  font-style: italic;
}
/* Comment.Special */
.code-highlight .gd {
  color: #A00000;
}
/* Generic.Deleted */
.code-highlight .ge {
  font-style: italic;
}
/* Generic.Emph */
.code-highlight .gr {
  color: #FF0000;
}
/* Generic.Error */
.code-highlight .gh {
  color: #000080;
  font-weight: bold;
}
/* Generic.Heading */
.code-highlight .gi {
  color: #00A000;
}
/* Generic.Inserted */
.code-highlight .go {
  color: #888888;
}
/* Generic.Output */
.code-highlight .gp {
  color: #000080;
  font-weight: bold;
}
/* Generic.Prompt */
.code-highlight .gs {
  font-weight: bold;
}
/* Generic.Strong */
.code-highlight .gu {
  color: #800080;
  font-weight: bold;
}
/* Generic.Subheading */
.code-highlight .gt {
  color: #0044DD;
}
/* Generic.Traceback */
.code-highlight .kc {
  color: #008000;
  font-weight: bold;
}
/* Keyword.Constant */
.code-highlight .kd {
  color: #008000;
  font-weight: bold;
}
/* Keyword.Declaration */
.code-highlight .kn {
  color: #008000;
  font-weight: bold;
}
/* Keyword.Namespace */
.code-highlight .kp {
  color: #008000;
}
/* Keyword.Pseudo */
.code-highlight .kr {
  color: #008000;
  font-weight: bold;
}
/* Keyword.Reserved */
.code-highlight .kt {
  color: #B00040;
}
/* Keyword.Type */
.code-highlight .m {
  color: #666666;
}
/* Literal.Number */
.code-highlight .s {
  color: #BA2121;
}
/* Literal.String */
.code-highlight .na {
  color: #7D9029;
}
/* Name.Attribute */
.code-highlight .nb {
  color: #008000;
}
/* Name.Builtin */
.code-highlight .nc {
  color: #0000FF;
  font-weight: bold;
}
/* Name.Class */
.code-highlight .no {
  color: #880000;
}
/* Name.Constant */
.code-highlight .nd {
  color: #AA22FF;
}
/* Name.Decorator */
.code-highlight .ni {
  color: #999999;
  font-weight: bold;
}
/* Name.Entity */
.code-highlight .ne {
  color: #D2413A;
  font-weight: bold;
}
/* Name.Exception */
.code-highlight .nf {
  color: #0000FF;
}
/* Name.Function */
.code-highlight .nl {
  color: #A0A000;
}
/* Name.Label */
.code-highlight .nn {
  color: #0000FF;
  font-weight: bold;
}
/* Name.Namespace */
.code-highlight .nt {
  color: #008000;
  font-weight: bold;
}
/* Name.Tag */
.code-highlight .nv {
  color: #19177C;
}
/* Name.Variable */
.code-highlight .ow {
  color: #AA22FF;
  font-weight: bold;
}
/* Operator.Word */
.code-highlight .w {
  color: #bbbbbb;
}
/* Text.Whitespace */
.code-highlight .mb {
  color: #666666;
}
/* Literal.Number.Bin */
.code-highlight .mf {
  color: #666666;
}
/* Literal.Number.Float */
.code-highlight .mh {
  color: #666666;
}
/* Literal.Number.Hex */
.code-highlight .mi {
  color: #666666;
}
/* Literal.Number.Integer */
.code-highlight .mo {
  color: #666666;
}
/* Literal.Number.Oct */
.code-highlight .sa {
  color: #BA2121;
}
/* Literal.String.Affix */
.code-highlight .sb {
  color: #BA2121;
}
/* Literal.String.Backtick */
.code-highlight .sc {
  color: #BA2121;
}
/* Literal.String.Char */
.code-highlight .dl {
  color: #BA2121;
}
/* Literal.String.Delimiter */
.code-highlight .sd {
  color: #BA2121;
  font-style: italic;
}
/* Literal.String.Doc */
.code-highlight .s2 {
  color: #BA2121;
}
/* Literal.String.Double */
.code-highlight .se {
  color: #BB6622;
  font-weight: bold;
}
/* Literal.String.Escape */
.code-highlight .sh {
  color: #BA2121;
}
/* Literal.String.Heredoc */
.code-highlight .si {
  color: #BB6688;
  font-weight: bold;
}
/* Literal.String.Interpol */
.code-highlight .sx {
  color: #008000;
}
/* Literal.String.Other */
.code-highlight .sr {
  color: #BB6688;
}
/* Literal.String.Regex */
.code-highlight .s1 {
  color: #BA2121;
}
/* Literal.String.Single */
.code-highlight .ss {
  color: #19177C;
}
/* Literal.String.Symbol */
.code-highlight .bp {
  color: #008000;
}
/* Name.Builtin.Pseudo */
.code-highlight .fm {
  color: #0000FF;
}
/* Name.Function.Magic */
.code-highlight .vc {
  color: #19177C;
}
/* Name.Variable.Class */
.code-highlight .vg {
  color: #19177C;
}
/* Name.Variable.Global */
.code-highlight .vi {
  color: #19177C;
}
/* Name.Variable.Instance */
.code-highlight .vm {
  color: #19177C;
}
/* Name.Variable.Magic */
.code-highlight .il {
  color: #666666;
}
/* Literal.Number.Integer.Long */
.searx-navbar {
  background: #eee;
  color: #aaa;
  height: 2.3rem;
  font-size: 1.3rem;
  line-height: 1.3rem;
  padding: 0.5rem;
  font-weight: bold;
  margin-bottom: 1.3rem;
}
.searx-navbar a,
.searx-navbar a:hover {
  margin-right: 2rem;
  text-decoration: none;
}
.searx-navbar .instance a {
  color: #444;
  margin-left: 2rem;
}
.table > tbody > tr > td,
.table > tbody > tr > th {
  vertical-align: middle !important;
}
.engine-tooltip {
  display: none;
  position: absolute;
  padding: 0.5rem 1rem;
  margin: 0rem 0 0 2rem;
  border: 1px solid #ddd;
  background: white;
  font-size: 14px;
  font-weight: normal;
  z-index: 1000000;
}
th:hover .engine-tooltip,
.engine-tooltip:hover {
  display: inline-block;
}
