@black: #29314D;
@gray: #A4A4A4;
@dim-gray: #F6F9FA;
@dark-gray: #666;
@semidark-gray: #888;
@middle-gray: #F5F5F5;
@mild-gray: #DDD;
@dark-blue: #007AB8;
@blue: #0088CC;
@red: #F35E77;
@violet: #684898;
@dark-green: #068922;
@green: #2ecc71;
@light-green: #01D7D4;
@orange: #FFA92F;
@dark-red: #c9432f;
