
.result_header {
    margin-bottom:5px;
    margin-top:20px;

    .favicon {
        margin-bottom:-3px;
    }

    a {
        vertical-align: bottom;

        .highlight {
            font-weight:bold;
        }
    }
}

.result-content {
    margin-top: 5px;
    word-wrap: break-word;

    .highlight {
        font-weight:bold;
    }
}

// default formating of results
.result-default {
    clear: both;
}

// image formating of results
.result-images {
    float: left !important;
    height: 138px;
}

.img-thumbnail {
    margin: 5px;
    max-height: 128px;
}

// video formating of results
.result-videos {
    clear: both;
}

// torrent formating of results
.result-torrents {
    clear: both;
}

// map formating of results
.result-map {
    clear: both;
}

// code formating of results
.result-code {
    clear: both;
}

// suggestion
.suggestion_item {
    margin: 2px 5px;
    max-width: 100%;

    .btn {
        max-width: 100%;
        white-space: normal;
        word-wrap: break-word;
        text-align: left;
    }
}

// download result
.result_download {
    margin-right: 5px;
}

// page forward, backward
#pagination {
    margin-top: 30px;
    padding-bottom: 50px;
}

.label-default {
    color: #AAA;
    background: #FFF;
}

.result .text-muted small {
    word-wrap: break-word;
}

.modal-wrapper {
    box-shadow: 0 5px 15px rgba(0, 0, 0, 0.5);
}

.modal-wrapper {
    background-clip: padding-box;
    background-color: #fff;
    border: 1px solid rgba(0, 0, 0, 0.2);
    border-radius: 6px;
    box-shadow: 0 3px 9px rgba(0, 0, 0, 0.5);
    outline: 0 none;
    position: relative;
}

.highlight {
    font-weight: 700;
}
