// Sticky footer styles
*{
  border-radius: 0 !important;
}
html {
  position: relative;
  min-height: 100%;
  color: @black;
}

body {
  /* Margin bottom by footer height */
  font-family: 'Roboto', Helvetica, Arial, sans-serif;
  margin-bottom: 80px;
  background-color: white;

  a{
      color: @blue;
  }
}

.footer {
  position: absolute;
  bottom: 0;
  width: 100%;
  /* Set the fixed height of the footer here */
  height: 60px;
  text-align: center;
  color: #999;
}
