{%- from 'oscar/macros.html' import draw_favicon with context -%}

<a href="{{ result.img_src }}" {% if results_on_new_tab %}target="_blank" rel="noopener noreferrer"{% else %}rel="noreferrer"{% endif %} data-toggle="modal" data-target="#modal-{{ index }}-{{pageno}}" id="result-{{loop.index}}">{{- "" -}}
    <img src="{% if result.thumbnail_src %}{{ image_proxify(result.thumbnail_src) }}{% else %}{{ image_proxify(result.img_src) }}{% endif %}" alt="{{ result.title|striptags }}" title="{{ result.title|striptags }}" class="img-thumbnail">{{- "" -}}
</a>
<div class="modal fade" id="modal-{{ index }}-{{ pageno }}" tabindex="-1" role="dialog" aria-hidden="true">{{- "" -}}
    <div class="modal-dialog">{{- "" -}}
        <div class="modal-wrapper">{{- "" -}}
            <div class="modal-header">{{- "" -}}
                <button type="button" class="close" data-dismiss="modal"><span aria-hidden="true">&times;</span><span class="sr-only">Close</span></button>{{- "" -}}
                <h4 class="modal-title">{% if result.engine~".png" in favicons %}{{ draw_favicon(result.engine) }} {% endif %}{{ result.title|striptags }}</h4>{{- "" -}}
            </div>{{- "" -}}
            <div class="modal-body">{{- "" -}}
                <img class="img-responsive center-block" src="{% if result.thumbnail_src %}{{ image_proxify(result.thumbnail_src) }}{% else %}{{ image_proxify(result.img_src) }}{% endif %}" alt="{{ result.title|striptags }}">
                {%- if result.author %}<span class="photo-author">{{ result.author }}</span><br />{% endif -%}
                {%- if result.content %}<p class="result-content">{{ result.content|striptags }}</p>{% endif -%}
                {%- if result.img_format %}<p class="result-format">{{ result.img_format }}</p>{% endif -%}
                {%- if result.source %}<p class="result-source">{{ result.source }}</p>{% endif -%}
            </div>{{- "" -}}
            <div class="modal-footer">{{- "" -}}
                <div class="clearfix"></div>{{- "" -}}
                <span class="label label-default pull-right">{{ result.engine }}</span>{{- "" -}}
                <p class="text-muted pull-left">{{ result.pretty_url }}</p>{{- "" -}}
                <div class="clearfix"></div>{{- "" -}}
                                <div class="row">{{- "" -}}
                    <div class="col-md-6">{{- "" -}}
                        <a href="{{ result.img_src }}" {% if results_on_new_tab %}target="_blank" rel="noopener noreferrer"{% else %}rel="noreferrer"{% endif %} class="btn btn-default">{{ _('Get image') }}</a>{{- "" -}}
                    </div>{{- "" -}}
                    <div class="col-md-6">{{- "" -}}
                        <a href="{{ result.url }}" {% if results_on_new_tab %}target="_blank" rel="noopener noreferrer"{% else %}rel="noreferrer"{% endif %} class="btn btn-default">{{ _('View source') }}</a>{{- "" -}}
                    </div>{{- "" -}}
                </div>{{- "" -}}
            </div>{{- "" -}}
        </div>{{- "" -}}
    </div>{{- "" -}}
</div>{{- "" -}}
