{% from 'simple/macros.html' import result_header, result_sub_header, result_sub_footer, result_footer, result_footer_rtl %}

{{ result_header(result, favicons, image_proxify) -}}
{{- result_sub_header(result) -}}
{% if result.embedded -%}
<p class="altlink"> &bull; <a class="btn-collapse collapsed media-loader disabled_if_nojs" data-target="#result-media-{{ index }}" data-btn-text-collapsed="{{ _('show media') }}" data-btn-text-not-collapsed="{{ _('hide media') }}">{{ icon('music-note') }} {{ _('show media') }}</a></p>
{%- endif %}
<p class="content">
  {%- if result.content %}{{ result.content|safe }}{% endif -%}
</p>
{{- result_sub_footer(result, proxify) -}}
{% if result.embedded -%}
<div id="result-media-{{ index }}" class="invisible">
   {{- result.embedded|safe -}}
</div>
{%- endif %}
{{- result_footer(result) }}
