.index {

  text-align: center;

  .title {
    background: url('../img/searx.png') no-repeat;
    width: 100%;
    min-height: 80px;
    background-position: center;
  }

  h1 {
    font-size: 5em;
    visibility: hidden;
  }

  #search {
    margin: 0 auto;
    background: inherit;
    border: inherit;
  }

  .search_filters {
    display: block;
    margin: 1em 0;
  }

  .category label {
    padding: 6px 10px;
    border-bottom: initial !important;
  }

}

@media screen and (max-width: 75em) {

  div.title {
    h1 {
      font-size: 1em;
    }
  }

}
