/*
 * searx, A privacy-respecting, hackable metasearch engine
 *
 * To change the colors of the site, simple edit this variables
 */

/// Basic Colors

@color-base: #3498DB;
@color-base-dark: #084999;
@color-base-light: #ECF0F1;
@color-highlight: #094089;
@color-black: #000000;

///
@color-error: #db3434;
@color-error-background: lighten(@color-error, 40%);

@color-warning: #dbba34;
@color-warning-background: lighten(@color-warning, 40%);

/// General

@color-font: #444;
@color-font-light: #888;

@color-red: #25a55b;

@color-url-font: #29314d;
@color-url-visited-font: #684898;
@results-width: 50em;
@search-width: 40em;

//
@color-a-font: @color-base;
@color-a-font-hover: @color-base;


/// Start-Screen



/// Search-Input

@color-search-border: @color-base;
@color-search-background: #FFF;
@color-search-font: #222;

/// Autocompleter

@color-autocompleter-choices-background: #FFF;
@color-autocompleter-choices-border: @color-base;
@color-autocompleter-choices-border-left-right: @color-base;
@color-autocompleter-choices-border-bottom: @color-base;

@color-autocompleter-choices-font: #444;

/// Answers
@color-answers-border: @color-base-dark;

// Selected
@color-autocompleter-selected-background: #444;
@color-autocompleter-selected-font: #FFF;
@color-autocompleter-selected-queried-font: #9FCFFF;

/// Categories

@color-categories-item-selected: @color-base;
@color-categories-item-selected-font: #FFF;

@color-categories-item-border-selected: @color-base-dark;
@color-categories-item-border-unselected: #E8E7E6;
@color-categories-item-border-unselected-hover: @color-base;


/// Results

@color-suggestions-button-background: @color-base;
@color-suggestions-button-font: #FFF;

@color-download-button-background: @color-base;
@color-download-button-font: #FFF;

@color-result-search-background: @color-base-light;

@color-result-definition-border: gray;
@color-result-torrent-border: lightgray;
@color-result-top-border: #E8E7E6;

// Link to result
@color-result-link-font: @color-base-dark;
@color-result-link-visited-font: @color-url-visited-font;

// Url to result
@color-result-url-font: @color-red;

// Publish Date
@color-result-publishdate-font: @color-font-light;

// Images
@color-result-image-span-background-hover: rgba(0, 0, 0, 0.6);
@color-result-image-span-font: #FFF;

// Search-URL
@color-result-search-url-border: #888;
@color-result-search-url-font: #444;


/// Settings

@color-settings-fieldset: @color-base;
@color-settings-table-striped: #dbdbdb;
@color-settings-tr-hover: #ececec;

// Labels
@color-settings-label-allowed-background: #E74C3C;
@color-settings-label-allowed-font: #FFF;

@color-settings-label-deny-background: #2ECC71;
@color-settings-label-deny-font: @color-font;

@color-settings-return-background: @color-base;
@color-settings-return-font: #FFF;

/// Other

@color-engines-font: @color-font-light;
@color-percentage-div-background: #444;

/// Load fonts from this directory.
@icon-font-path:          "../../../fonts/";
//** File name for all font files.
@icon-font-name:          "glyphicons-halflings-regular";
//** Element ID within SVG icon file.
@icon-font-svg-id:        "glyphicons_halflingsregular";
