{% from 'simple/macros.html' import icon, tabs_open, tabs_close, tab_header, tab_footer, checkbox_onoff, checkbox %}

{% extends "simple/base.html" %}

{%- macro plugin_preferences(section) -%}
{%- for plugin in plugins -%}
{%- if plugin.preference_section == section -%}
<fieldset>{{- '' -}}
    <legend>{{ _(plugin.name) }}</legend>{{- '' -}}
    <div class="value">
      {{- checkbox_onoff('plugin_' + plugin.id, plugin.id not in allowed_plugins) -}}
    </div>{{- '' -}}
    <div class="description">
      {{- _(plugin.description) -}}
    </div>{{- '' -}}
</fieldset>
{%- endif -%}
{%- endfor -%}
{%- endmacro -%}

{% macro engine_about(search_engine) -%}
{% if search_engine.about is defined %}
{% set about = search_engine.about %}
<div class="engine-tooltip" role="tooltip">{{- "" -}}
    <p><a href="{{about.website}}" rel="noreferrer">{{about.website}}</a></p>
    {%- if about.wikidata_id -%}<p><a href="https://www.wikidata.org/wiki/{{about.wikidata_id}}" rel="noreferrer">wikidata.org/wiki/{{about.wikidata_id}}</a></p>{%- endif -%}
    {%- if search_engine.enable_http %}<p>{{ icon('exclamation-sign', 'No HTTPS') }}{{ _('No HTTPS')}}</p>{% endif -%}
</div>
{%- endif -%}
{%- endmacro %}

{% block head %} {% endblock %}
{% block content %}

<a href="{{ url_for('index') }}"><h1><span>searx</span></h1></a>

<h2>{{ _('Preferences') }}</h2>

<form id="search_form" method="post" action="{{ url_for('preferences') }}">

{{ tabs_open() }}

{{ tab_header('maintab', 'general', _('General')) }}
  {% if 'categories' not in locked_preferences %}
  <fieldset>
    <legend>{{ _('Default categories') }}</legend>
    {% set display_tooltip = false %}
    {% include 'simple/categories.html' %}
  </fieldset>
  {% endif %}
  {% if 'language' not in locked_preferences %}
  <fieldset>
    <legend>{{ _('Search language') }}</legend>
    <p class="value">{{- '' -}}
      <select name='language'>{{- '' -}}
        <option value="all" {% if current_language == 'all' %}selected="selected"{% endif %}>{{ _('Default language') }}</option>
        {%- for lang_id,lang_name,country_name,english_name in language_codes | sort(attribute=1) -%}
        <option value="{{ lang_id }}" {% if lang_id == current_language %}selected="selected"{% endif %}>{{ lang_name }} {% if country_name %}({{ country_name }}) {% endif %}- {{ lang_id }}</option>
        {%- endfor -%}
      </select>{{- '' -}}
    </p>
    <div class="description">{{ _('What language do you prefer for search?') }}</div>
  </fieldset>
  {% endif %}
  {% if 'autocomplete' not in locked_preferences %}
  <fieldset>
    <legend>{{ _('Autocomplete') }}</legend>
    <p class="value">
      <select name="autocomplete">
        <option value=""> - </option>
        {%- for backend in autocomplete_backends -%}
        <option value="{{ backend }}" {% if backend == autocomplete %}selected="selected"{% endif %}>{{ backend }}</option>
        {%- endfor -%}
      </select>
    </p>
    <div class="description">{{ _('Find stuff as you type') }}</div>
  </fieldset>
  {% endif %}
  {% if 'safesearch' not in locked_preferences %}
  <fieldset>
    <legend>{{ _('SafeSearch') }}</legend>
    <p class="value">
      <select name='safesearch'>
        <option value="2" {% if safesearch == '2' %}selected="selected"{% endif %}>{{ _('Strict') }}</option>
        <option value="1" {% if safesearch == '1' %}selected="selected"{% endif %}>{{ _('Moderate') }}</option>
        <option value="0" {% if safesearch == '0' %}selected="selected"{% endif %}>{{ _('None') }}</option>
      </select>
    </p>
    <p class="description">{{ _('Filter content') }}</p>
  </fieldset>
  {% endif %}
  {{ plugin_preferences('general') }}
  {% if 'doi_resolver' not in locked_preferences %}
  <fieldset>
    <legend>{{ _('Open Access DOI resolver') }}</legend>
    <p class="value">
      <select id='doi_resolver' name='doi_resolver'>
      {%- for doi_resolver_name,doi_resolver_url in doi_resolvers.items() -%}
         <option value="{{ doi_resolver_name }}" {% if doi_resolver_url == current_doi_resolver %}selected="selected"{% endif %}>
         {{- doi_resolver_name }} - {{ doi_resolver_url -}}
         </option>
      {%- endfor -%}
      </select>
    </p>
    <div class="description"><!-- {{ _('Redirect to open-access versions of publications when available (plugin required)') }} --></div>
  </fieldset>
  {% endif %}
  {{ tab_footer() }}

  {{ tab_header('maintab', 'engines', _('Engines')) }}
    <p>{{ _('Currently used search engines') }}</p>

    {{ tabs_open() }}
    {% for categ in all_categories %}

    {{ tab_header('enginetab', 'category' + categ, _(categ)) }}
    <div class="scrollx">
    <table class="striped">
      <tr>
        <th class="engine_checkbox">{{ _("Allow") }}</th>
        <th class="name">{{ _("Engine name") }}</th>
        <th class="shortcut">{{ _("Shortcut") }}</th>
        <th>{{ _("Supports selected language") }}</th>
        <th>{{ _("SafeSearch") }}</th>
        <th>{{ _("Time range") }}</th>
        <th>{{ _("Avg. time") }}</th>
        <th>{{ _("Max time") }}</th>
      </tr>
      {% for search_engine in engines_by_category[categ] %}

      {% if not search_engine.private %}
      {% set engine_id = 'engine_' + search_engine.name|replace(' ', '_') + '__' + categ|replace(' ', '_') %}
      <tr>
        <td class="engine_checkbox">{{ checkbox_onoff(engine_id, (search_engine.name, categ) in disabled_engines) }}</td>
        <th class="name">{% if search_engine.enable_http %}{{ icon('warning', 'No HTTPS') }}{% endif %} {{ search_engine.name }} {{ engine_about(search_engine) }}</th>
        <td class="shortcut">{{ shortcuts[search_engine.name] }}</td>
        <td>{{ checkbox(engine_id + '_supported_languages', current_language == 'all' or current_language in search_engine.supported_languages or current_language.split('-')[0] in search_engine.supported_languages, true, true) }}</td>
        <td>{{ checkbox(engine_id + '_safesearch', search_engine.safesearch==True, true, true) }}</td>
        <td>{{ checkbox(engine_id + '_time_range_support', search_engine.time_range_support==True, true, true) }}</td>
        <td class="{{ 'danger' if stats[search_engine.name]['warn_time'] else '' }}">{{ 'N/A' if stats[search_engine.name].time==None else stats[search_engine.name].time }}</td>
        <td class="{{ 'danger' if stats[search_engine.name]['warn_timeout'] else '' }}">{{ search_engine.timeout }}</td>
      </tr>
      {% endif %}
      {% endfor %}
    </table>
    </div>
    {{ tab_footer() }}
    {% endfor %}
    {{ tabs_close() }}
  {{ tab_footer() }}

  {{ tab_header('maintab', 'ui', _('User interface')) }}
  {% if 'locale' not in locked_preferences %}
  <fieldset>
    <legend>{{ _('Interface language') }}</legend>
    <p class="value">
      <select name='locale'>
        {% for locale_id,locale_name in locales.items() | sort %}
        <option value="{{ locale_id }}" {% if locale_id == current_locale %}selected="selected"{% endif %}>{{ locale_name }}</option>
        {% endfor %}
      </select>
    </p>
    <div class="description">{{ _('Change the language of the layout') }}</div>
  </fieldset>
  {% endif %}
  {% if 'theme' not in locked_preferences %}
  <fieldset>
    <legend>{{ _('Themes') }}</legend>
    <p class="value">
      <select name="theme">
        {%- for name in themes -%}
        <option value="{{ name }}" {% if name == theme %}selected="selected"{% endif %}>{{ name }}</option>
        {%- endfor -%}
      </select>
    </p>
    <div class="description">{{ _('Change searx layout') }}</div>
  </fieldset>
  {% endif %}
  {% if 'results_on_new_tab' not in locked_preferences %}
  <fieldset>
    <legend>{{ _('Results on new tabs') }}</legend>
    <p class="value">
      <select name='results_on_new_tab'>
        <option value="1" {% if results_on_new_tab %}selected="selected"{% endif %}>{{ _('On') }}</option>
        <option value="0" {% if not results_on_new_tab %}selected="selected"{% endif %}>{{ _('Off')}}</option>
      </select>
    </p>
    <div class="description">{{_('Open result links on new browser tabs') }}</div>
  </fieldset>
  {% endif %}
  {% if 'autofocus' not in locked_preferences %}
  <fieldset>
    <legend>{{ _('Autofocus search field') }}</legend>
    <p class="value">
      <select name='autofocus'>
        <option value="1" {% if autofocus %}selected="selected"{% endif %}>{{ _('On') }}</option>
        <option value="0" {% if not autofocus %}selected="selected"{% endif %}>{{ _('Off')}}</option>
      </select>
    </p>
    <div class="description">{{_('Turn off if you use your keyboard to scroll') }}</div>
  </fieldset>
  {% endif %}
  {{ plugin_preferences('ui') }}
  {{ tab_footer() }}

  {{ tab_header('maintab', 'cookies', _('Cookies')) }}

     <p class="text-muted" style="margin:20px 0;">
        {{ _('This is the list of cookies and their values searx is storing on your computer.') }}<br />
        {{ _('With that list, you can assess searx transparency.') }}<br />
     </p>

     {% if cookies %}
     <table class="cookies">
       <tr>
          <th>{{ _('Cookie name') }}</th>
          <th>{{ _('Value') }}</th>
       </tr>
       {% for cookie in cookies %}
       <tr>
          <td>{{ cookie }}</td>
          <td>{{ cookies[cookie] }}</td>
       </tr>
       {% endfor %}
     </table>
     {% else %}
        {% include 'oscar/messages/no_cookies.html' %}
     {% endif %}

     <h4>{{ _('Search URL of the currently saved preferences') }} :</h4>
     <div class="selectable_url">
       <pre>{{ url_for('index', _external=True) }}?preferences={{ preferences_url_params|e }}{% raw %}&amp;q=%s{% endraw %}</pre>
     </div>
     <p class="small_font">{{ _('Note: specifying custom settings in the search URL can reduce privacy by leaking data to the clicked result sites.') }}</p>

  {{ tab_footer() }}

  {{ tab_header('maintab', 'privacy', _('Privacy')) }}
  {% if 'method' not in locked_preferences %}
  <fieldset>
    <legend>{{ _('Method') }}</legend>
    <p class="value">
      <select name='method'>
        <option value="POST" {% if method == 'POST' %}selected="selected"{% endif %}>POST</option>
        <option value="GET" {% if method == 'GET' %}selected="selected"{% endif %}>GET</option>
      </select>
    </p>
    <div class="description">{{ _('Search language') }}</div>
  </fieldset>
  {% endif %}
  {% if 'image_proxy' not in locked_preferences %}
  <fieldset>
    <legend>{{ _('Image proxy') }}</legend>
    <p class="value">
      <select name='image_proxy'>
        <option value="1" {% if image_proxy %}selected="selected"{% endif %}>{{ _('Enabled') }}</option>
        <option value="" {% if not image_proxy %}selected="selected"{% endif %}>{{ _('Disabled') }}</option>
      </select>
    </p>
    <div class="description">{{ _('Proxying image results through searx') }}</div>
  </fieldset>
  {% endif %}
  {{ plugin_preferences('privacy') }}

  {{ tab_footer() }}

  {{ tabs_close() }}

  <p class="small_font">{{ _('These settings are stored in your cookies, this allows us not to store this data about you.') }}
    <br />
    {{ _("These cookies serve your sole convenience, we don't use these cookies to track you.") }}
  </p>

  <input type="submit" value="{{ _('save') }}" />
  <div class="{% if rtl %}left{% else %}right{% endif %} preferences_back"><a href="{{ url_for('clear_cookies') }}">{{ _('Reset defaults') }}</a></div>
  <div class="{% if rtl %}left{% else %}right{% endif %} preferences_back"><a href="{{ url_for('index') }}">{{ _('back') }}</a></div>

</form>
{% endblock %}
