{% from 'oscar/macros.html' import icon %}
<div class="alert alert-danger fade in" role="alert">
    <button class="close" data-dismiss="alert" type="button">
        <span aria-hidden="true">×</span>
        <span class="sr-only">{{ _('Close') }}</span>
    </button>
    <strong class="lead">{{ icon('exclamation-sign') }} {{ _('Oh snap!') }}</strong>
    {{ _('Something went wrong.') }}
</div>
