#advanced-search-container {
    display: none;
    text-align: center;
    margin-bottom: 1rem;
    clear: both;

    label, .input-group-addon {
        font-size: 1.3rem;
        font-weight:normal;
        background-color: white;
        border: #DDD 1px solid;
        border-right: none;
        color: #333;
        padding-bottom: 0.8rem;
        padding-left: 1.2rem;
        padding-right: 1.2rem;
    }

    label:last-child, .input-group-addon:last-child {
        border-right: #DDD 1px solid;
    }

    input[type="radio"] {
        display: none;
    }

    input[type="radio"]:checked + label {
        color: black;
        font-weight: bold;
        background-color: #EEE;
    }
}

#check-advanced {
    display: none;
}

#check-advanced:checked ~ #advanced-search-container {
    display: block;
}

.advanced {
    padding: 0;
    margin-top: 0.3rem;
    text-align: right;
    label, select {
        cursor: pointer;
    }
}
