# -*- coding: utf-8 -*-
# list of language codes
# this file is generated automatically by utils/fetch_languages.py
language_codes = \
(   ('af-ZA', 'Afrikaans', '', 'Afrikaans'),
    ('ar-EG', 'العربية', '', 'Arabic'),
    ('be-BY', 'Беларуская', '', 'Belarusian'),
    ('bg-BG', 'Български', '', 'Bulgarian'),
    ('ca-ES', 'Català', '', 'Catalan'),
    ('cs-CZ', 'Čeština', '', 'Czech'),
    ('da-DK', 'Dansk', '', 'Danish'),
    ('de', 'Deutsch', '', 'German'),
    ('de-AT', 'Deutsch', 'Österreich', 'German'),
    ('de-CH', 'Deutsch', 'Schweiz', 'German'),
    ('de-DE', 'Deutsch', 'Deutschland', 'German'),
    ('el-GR', 'Ελληνικά', '', 'Greek'),
    ('en', 'English', '', 'English'),
    ('en-AU', 'English', 'Australia', 'English'),
    ('en-CA', 'English', 'Canada', 'English'),
    ('en-GB', 'English', 'United Kingdom', 'English'),
    ('en-IE', 'English', 'Ireland', 'English'),
    ('en-IN', 'English', 'India', 'English'),
    ('en-NZ', 'English', 'New Zealand', 'English'),
    ('en-US', 'English', 'United States', 'English'),
    ('es', 'Español', '', 'Spanish'),
    ('es-AR', 'Español', 'Argentina', 'Spanish'),
    ('es-CL', 'Español', 'Chile', 'Spanish'),
    ('es-ES', 'Español', 'España', 'Spanish'),
    ('es-MX', 'Español', 'México', 'Spanish'),
    ('et-EE', 'Eesti', '', 'Estonian'),
    ('fa-IR', 'فارسی', '', 'Persian'),
    ('fi-FI', 'Suomi', '', 'Finnish'),
    ('fr', 'Français', '', 'French'),
    ('fr-BE', 'Français', 'Belgique', 'French'),
    ('fr-CA', 'Français', 'Canada', 'French'),
    ('fr-CH', 'Français', 'Suisse', 'French'),
    ('fr-FR', 'Français', 'France', 'French'),
    ('he-IL', 'עברית', '', 'Hebrew'),
    ('hr-HR', 'Hrvatski', '', 'Croatian'),
    ('hu-HU', 'Magyar', '', 'Hungarian'),
    ('hy-AM', 'Հայերեն', '', 'Armenian'),
    ('id-ID', 'Indonesia', '', 'Indonesian'),
    ('is-IS', 'Íslenska', '', 'Icelandic'),
    ('it-IT', 'Italiano', '', 'Italian'),
    ('ja-JP', '日本語', '', 'Japanese'),
    ('ko-KR', '한국어', '', 'Korean'),
    ('lt-LT', 'Lietuvių', '', 'Lithuanian'),
    ('lv-LV', 'Latviešu', '', 'Latvian'),
    ('nb-NO', 'Norsk Bokmål', '', 'Norwegian Bokmål'),
    ('nl', 'Nederlands', '', 'Dutch'),
    ('nl-BE', 'Nederlands', 'België', 'Dutch'),
    ('nl-NL', 'Nederlands', 'Nederland', 'Dutch'),
    ('pl-PL', 'Polski', '', 'Polish'),
    ('pt', 'Português', '', 'Portuguese'),
    ('pt-BR', 'Português', 'Brasil', 'Portuguese'),
    ('pt-PT', 'Português', 'Portugal', 'Portuguese'),
    ('ro-RO', 'Română', '', 'Romanian'),
    ('ru-RU', 'Русский', '', 'Russian'),
    ('sk-SK', 'Slovenčina', '', 'Slovak'),
    ('sl-SI', 'Slovenščina', '', 'Slovenian'),
    ('sr-RS', 'Srpski', '', 'Serbian'),
    ('sv-SE', 'Svenska', '', 'Swedish'),
    ('sw-TZ', 'Kiswahili', '', 'Swahili'),
    ('th-TH', 'ไทย', '', 'Thai'),
    ('tr-TR', 'Türkçe', '', 'Turkish'),
    ('uk-UA', 'Українська', '', 'Ukrainian'),
    ('vi-VN', 'Tiếng Việt', '', 'Vietnamese'),
    ('zh', '中文', '', 'Chinese'),
    ('zh-CN', '中文', '中国', 'Chinese'),
    ('zh-HK', '中文', '中國香港特別行政區', 'Chinese'),
    ('zh-TW', '中文', '台灣', 'Chinese'))