<!DOCTYPE html>
<html class="no-js" lang="en" {% if rtl %} dir="rtl"{% endif %}>
<head>
  <meta charset="UTF-8" />
  <meta name="description" content="searx — a privacy-respecting, hackable metasearch engine">
  <meta name="keywords" content="searx, search, search engine, metasearch, meta search">
  <meta name="generator" content="searx/{{ searx_version }}">
  <meta name="referrer" content="no-referrer">
  <meta name="robots" content="noarchive">
  <meta name="viewport" content="width=device-width, maximum-scale=1.0, user-scalable=1">
  <meta name="HandheldFriendly" content="True">
  <meta http-equiv="X-UA-Compatible" content="IE=edge, chrome=1">
  <title>{% block title %}{% endblock %}{{ instance_name }}</title>
  {% block meta %}{% endblock %}
  {% if rtl %}
  <link rel="stylesheet" href="{{ url_for('static', filename='css/searx-rtl.min.css') }}" type="text/css" media="screen" />
  {% else %}
  <link rel="stylesheet" href="{{ url_for('static', filename='css/searx.min.css') }}" type="text/css" media="screen" />
  {% endif %}
  {% block styles %}{% endblock %}
  <!--[if gte IE 9]>-->
  <script src="{{ url_for('static', filename='js/searx.head.min.js') }}"
          data-method="{{ method or 'POST' }}"
          data-autocompleter="{% if autocomplete %}true{% else %}false{% endif %}"
          data-search-on-category-select="{{ 'true' if 'plugins/js/search_on_category_select.js' in scripts else 'false'}}"
          data-infinite-scroll="{{ 'true' if 'plugins/js/infinite_scroll.js' in scripts else 'false' }}"
          data-static-path="{{ url_for('static', filename='themes/simple') }}/"
          data-translations="{{ translations }}"></script>
  <!--<![endif]-->
  {% block head %}
  <link title="{{ instance_name }}" type="application/opensearchdescription+xml" rel="search" href="{{ opensearch_url }}"/>
  {% endblock %}
  <link rel="shortcut icon" href="{{ url_for('static', filename='img/favicon.png') }}" />
</head>
<body class="{{ endpoint }}_endpoint" >
  <main id="main_{{  self._TemplateReference__context.name|replace("simple/", "")|replace(".html", "") }}">
    {% if errors %}
        <div class="dialog-error" role="alert">
            <a href="#" class="close" aria-label="close" title="close">×</a>
            <ul>
            {% for message in errors %}
                <li>{{ message }}</li>
            {% endfor %}
            </ul>
        </div>
    {% endif %}

    {% block content %}
    {% endblock %}
  </main>
  <footer>
    <p>
    {{ _('Powered by') }} <a href="{{ url_for('about') }}">searx</a> - {{ searx_version }} — {{ _('a privacy-respecting, hackable metasearch engine') }}<br/>
        <a href="{{ brand.GIT_URL }}">{{ _('Source code') }}</a> |
        <a href="{{ brand.ISSUE_URL }}">{{ _('Issue tracker') }}</a> |
        <a href="{{ brand.PUBLIC_INSTANCES }}">{{ _('Public instances') }}</a>{% if brand.CONTACT_URL %} |
        <a href="{{ brand.CONTACT_URL }}">{{ _('Contact instance maintainer') }}</a>{% endif %}
    </p>
  </footer>
  <!--[if gte IE 9]>-->
  <script src="{{ url_for('static', filename='js/searx.min.js') }}"></script>
  <!--<![endif]-->
</body>
</html>
